@@ -1,7 +1,4 @@
-try:
-    from collections.abc import Iterable
-except ImportError:  # For python 2.7 support
-    from collections import Iterable
+from collections.abc import Iterable
 
 import six
 import sqlalchemy as sa
@@ -13,6 +10,7 @@ from sqlalchemy.util import set_creation_order
 
 from .exceptions import ImproperlyConfigured
 from .functions import identity
+from .functions.orm import _get_class_registry
 
 
 class GenericAttributeImpl(attributes.ScalarAttributeImpl):
@@ -30,7 +28,7 @@ class GenericAttributeImpl(attributes.ScalarAttributeImpl):
         # Find class for discriminator.
         # TODO: Perhaps optimize with some sort of lookup?
         discriminator = self.get_state_discriminator(state)
-        target_class = state.class_._decl_class_registry.get(discriminator)
+        target_class = _get_class_registry(state.class_).get(discriminator)
 
         if target_class is None:
             # Unknown discriminator; return nothing.
@@ -112,7 +110,7 @@ class GenericRelationshipProperty(MapperProperty):
                 if key == attr_key:
                     return attr
         else:
-            for key, attr in self.parent.attrs.items():
+            for attr in self.parent.attrs.values():
                 if isinstance(attr, ColumnProperty):
                     if attr.columns[0].name == column.name:
                         return attr
